package com.upay.demo.declaration;

import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.MerchantInfoChangeBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.MerchantInfoChangeBusinessInfo;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.MerchantInfoChangeContactInfo;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.MerchantInfoChangeLegalPersonInfo;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.MerchantInfoChangeSubjectInfo;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.MerchantInfoChangeExecuter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 修改商户信息接口
 */
public class MerchantInfoChangeServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String partnerId = req.getParameter("partnerId");
        String merchantId = req.getParameter("merchantId");
        String requestId = req.getParameter("requestId");
        String notifyUrl = req.getParameter("notifyUrl");

        //请求参数（merchantSubjectInfo，商户主体信息）
        String signedShorthand = req.getParameter("signedShorthand");
        String businessLicensePath = req.getParameter("businessLicensePath");

        //请求参数（businessInfo，经营信息）
        String businessClassification = req.getParameter("businessClassification");
        String businessAddressProvince = req.getParameter("businessAddressProvince");
        String businessAddressCity = req.getParameter("businessAddressCity");
        String businessAddressArea = req.getParameter("businessAddressArea");
        String businessAddress = req.getParameter("businessAddress");
        String storeHeaderPath = req.getParameter("storeHeaderPath");
        String storeIndoorPath = req.getParameter("storeIndoorPath");

        //请求参数（legalPersonInfo，法人信息）
        String legalPersonIdType = req.getParameter("legalPersonIdType");
        String legalPersonName = req.getParameter("legalPersonName");
        String legalPersonIdNo = req.getParameter("legalPersonIdNo");
        String legalIdCardProsPath = req.getParameter("legalIdCardProsPath");
        String legalIdCardConsPath = req.getParameter("legalIdCardConsPath");
        String idEffectiveDateStart = req.getParameter("idEffectiveDateStart");
        String idEffectiveDateEnd = req.getParameter("idEffectiveDateEnd");

        //请求参数（contactInfo，商户联系人信息）
        String contactName = req.getParameter("contactName");
        String contactEmail = req.getParameter("contactEmail");
        String contactPhone = req.getParameter("contactPhone");

        MerchantInfoChangeBuilder builder = new MerchantInfoChangeBuilder();
        builder.setPartnerId(partnerId);
        builder.setMerchantId(merchantId);
        builder.setRequestId(requestId);
        builder.setNotifyUrl(notifyUrl);

        MerchantInfoChangeSubjectInfo merchantSubjectInfo = new MerchantInfoChangeSubjectInfo();
        merchantSubjectInfo.setSignedShorthand(signedShorthand);
        merchantSubjectInfo.setBusinessLicensePath(businessLicensePath);

        //请求参数（legalPersonInfo，法人信息）
        MerchantInfoChangeBusinessInfo businessInfo = new MerchantInfoChangeBusinessInfo();
        businessInfo.setBusinessClassification(businessClassification);
        businessInfo.setBusinessAddressProvince(businessAddressProvince);
        businessInfo.setBusinessAddressCity(businessAddressCity);
        businessInfo.setBusinessAddressArea(businessAddressArea);
        businessInfo.setBusinessAddress(businessAddress);
        businessInfo.setStoreHeaderPath(storeHeaderPath);
        businessInfo.setStoreIndoorPath(storeIndoorPath);
        //请求参数（legalPersonInfo，法人信息）
        MerchantInfoChangeLegalPersonInfo legalPersonInfo = new MerchantInfoChangeLegalPersonInfo();
        legalPersonInfo.setLegalPersonIdType(legalPersonIdType);
        legalPersonInfo.setLegalPersonName(legalPersonName);
        legalPersonInfo.setLegalPersonIdNo(legalPersonIdNo);
        legalPersonInfo.setLegalIdCardProsPath(legalIdCardProsPath);
        legalPersonInfo.setLegalIdCardConsPath(legalIdCardConsPath);
        legalPersonInfo.setIdEffectiveDateStart(idEffectiveDateStart);
        legalPersonInfo.setIdEffectiveDateEnd(idEffectiveDateEnd);

        //请求参数（contactInfo，商户联系人信息）
        MerchantInfoChangeContactInfo contactInfo = new MerchantInfoChangeContactInfo();
        contactInfo.setContactName(contactName);
        contactInfo.setContactEmail(contactEmail);
        contactInfo.setContactPhone(contactPhone);

        builder.setBusinessInfo(businessInfo);
        builder.setLegalPersonInfo(legalPersonInfo);
        builder.setContactInfo(contactInfo);

        final PrintWriter out = resp.getWriter();

        MerchantInfoChangeExecuter executer = new MerchantInfoChangeExecuter();

        try {

            executer.merchantInfoChange(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }
            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
