package com.upay.demo.declaration;


import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.ModifyBankCardBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.ModifyBankCardExecuter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ModifyBankCardServlet extends HttpServlet {

    static final Logger LOGGER = LoggerFactory.getLogger(ModifyBankCardServlet.class);

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
        // 请求信息
        String merchantId = req.getParameter("merchantId");
        String requestId = req.getParameter("requestId");
        String subMerchantId = req.getParameter("subMerchantId");
        String signedRequestId = req.getParameter("signedRequestId");

        ModifyBankCardBuilder builder = new ModifyBankCardBuilder(merchantId);
        builder.setRequestId(requestId);
        builder.setSignedRequestId(signedRequestId);
        builder.setSubMerchantId(subMerchantId);

        // 结算信息
        String bankName = req.getParameter("bankName");
        String bankBranchName = req.getParameter("bankBranchName");
        String bankCardNo = req.getParameter("bankCardNo");
        String provinceCode = req.getParameter("provinceCode");
        String cityCode = req.getParameter("cityCode");

        builder.setBankName(bankName);
        builder.setBankBranchName(bankBranchName);
        builder.setBankCardNo(bankCardNo);
        builder.setCityCode(cityCode);
        builder.setProvinceCode(provinceCode);

        final PrintWriter out = resp.getWriter();
        ModifyBankCardExecuter executer = new ModifyBankCardExecuter();

        try {
            executer.modify(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("请求成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toString();
                    out.print(result);
                }

            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
