package com.upay.demo.declaration;

import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.FastJsonUtils;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.DeclarationExecuter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 审核完成通知
 */
public class NotifyServlet extends HttpServlet {

    static final Logger LOGGER = LoggerFactory.getLogger(NotifyServlet.class);

    private static final String ENCRYPT_KEY = "encryptKey";
    private static final String MERCHANT_ID = "merchantId";

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        LOGGER.info("服务商入网审核完成回调");
        DeclarationExecuter executer = new DeclarationExecuter();
        try {
            JSONObject json = FastJsonUtils.convert(req.getInputStream());
            json.put(ENCRYPT_KEY, req.getHeader(ENCRYPT_KEY));
            json.put(MERCHANT_ID, req.getHeader(MERCHANT_ID));
            executer.callBack(json, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    LOGGER.info("服务商入网审核完成回调内容: {}", jsonObject.toJSONString());
                }
            });
        } catch (HmacVerifyException e) {
            LOGGER.info("服务商入网审核完成回调签名验证异常", e);
            LOGGER.info(e.toString());
        } catch (UnknownException e) {
            LOGGER.info("服务商入网审核完成回调未知异常", e);
            LOGGER.info(e.toString());
        }

        LOGGER.info("ReviewCompleteNotifyServlet notify success...");
        PrintWriter out = resp.getWriter();
        out.print("SUCCESS");
    }
}
