package com.upay.demo.declaration;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListener;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.PreDeclarationBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.ContactInfo;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.PaymentProfile;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.PreDeclarationExecuter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

/**
 * 二级商户银联报备接口
 */
public class PreDeclareServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String merchantId = req.getParameter("merchantId");
        String requestId = req.getParameter("requestId");
        String notifyUrl = req.getParameter("notifyUrl");
        String sendActiveEmail = req.getParameter("sendActiveEmail");
        String productPackage = req.getParameter("productPackage");
        String registerRole = req.getParameter("registerRole");
        String contactName = req.getParameter("contactName");
        String contactEmail = req.getParameter("contactEmail");
        String contactPhone = req.getParameter("contactPhone");
        String paymentMode = req.getParameter("paymentMode");
        String feeType = req.getParameter("feeType");
        String feeAmount = req.getParameter("feeAmount");

        PreDeclarationBuilder builder = new PreDeclarationBuilder();
        builder.setMerchantId(merchantId);
        builder.setRequestId(requestId);
        builder.setNotifyUrl(notifyUrl);
        builder.setSendActiveEmail(sendActiveEmail);
        builder.setProductPackage(productPackage);
        builder.setRegisterRole(registerRole);

        ContactInfo contactInfo = new ContactInfo();
        contactInfo.setContactName(contactName);
        contactInfo.setContactEmail(contactEmail);
        contactInfo.setContactPhone(contactPhone);

        //paymentProfiles_产品支付方式
        PaymentProfile paymentProfile = new PaymentProfile();
        paymentProfile.setPaymentMode(paymentMode);
        paymentProfile.setFeeType(feeType);
        paymentProfile.setFeeAmount(feeAmount);
        List<PaymentProfile> paymentProfiles = new ArrayList<>();
        paymentProfiles.add(paymentProfile);
        if (paymentMode.equals("") && paymentMode!=null){

        }else{
            builder.setPaymentProfiles(paymentProfiles);
        }


        final PrintWriter out = resp.getWriter();

        PreDeclarationExecuter executer = new PreDeclarationExecuter();

        try {
            executer.preDeclare(builder, new ResultListener() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void processing(JSONObject jsonObject) {

                }

                @Override
                public void redirect(JSONObject jsonObject, String s) throws IOException {

                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }
            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
