package com.upay.demo.declaration;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.DeclarationBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.entity.*;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.DeclarationExecuter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

public class ProductPackageDeclarationServlet extends HttpServlet {

    static final Logger LOGGER = LoggerFactory.getLogger(ProductPackageDeclarationServlet.class);

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {
        //基本信息
        String merchantId = req.getParameter("merchantId");
        String requestId = req.getParameter("requestId");
        String operationType = req.getParameter("operationType");
        String notifyUrl = req.getParameter("notifyUrl");
        String extendedParameters = req.getParameter("extendedParameters");

        DeclarationBuilder builder = new DeclarationBuilder(merchantId);
        builder.setRequestId(requestId);
        builder.setOperationType(operationType);
        builder.setNotifyUrl(notifyUrl);
        builder.setExtendedParameters(extendedParameters);

        //baseInfo_商户基本信息
        String signedType = req.getParameter("signedType");
        String signedName = req.getParameter("signedName");
        String registerRole = req.getParameter("registerRole");
        String cerType = req.getParameter("cerType");
        String signedShorthand = req.getParameter("signedShorthand");
        String businessAddressProvince = req.getParameter("businessAddressProvince");
        String businessAddressCity = req.getParameter("businessAddressCity");
        String businessAddressArea = req.getParameter("businessAddressArea");
        String businessAddress = req.getParameter("businessAddress");
        String contactName = req.getParameter("contactName");
        String contactEmail = req.getParameter("contactEmail");
        String contactPhone = req.getParameter("contactPhone");
        String businessClassification = req.getParameter("businessClassification");
        String desireAuth = req.getParameter("desireAuth");
        String belongAgentNumber = req.getParameter("belongAgentNumber");

        BaseInfo baseInfo = new BaseInfo();
        baseInfo.setSignedType(signedType);
        baseInfo.setSignedName(signedName);
        baseInfo.setRegisterRole(registerRole);
        baseInfo.setCerType(cerType);
        baseInfo.setSignedShorthand(signedShorthand);
        baseInfo.setBusinessAddressProvince(businessAddressProvince);
        baseInfo.setBusinessAddressCity(businessAddressCity);
        baseInfo.setBusinessAddressArea(businessAddressArea);
        baseInfo.setBusinessAddress(businessAddress);
        baseInfo.setContactName(contactName);
        baseInfo.setContactEmail(contactEmail);
        baseInfo.setContactPhone(contactPhone);
        baseInfo.setBusinessClassification(businessClassification);
        baseInfo.setDesireAuth(desireAuth);
        baseInfo.setBelongAgentNumber(belongAgentNumber);
        builder.setBaseInfo(baseInfo);

        //bankCardInfo_结算信息
        String bankName = req.getParameter("bankName");
        String bankBranchName = req.getParameter("bankBranchName");
        String accountName = req.getParameter("accountName");
        String bankCardNo = req.getParameter("bankCardNo");
        String cityCode = req.getParameter("cityCode");
        String provinceCode = req.getParameter("provinceCode");
        String accountType = req.getParameter("accountType");
        String liquidationType = req.getParameter("liquidationType");
        String withdrawRateType = req.getParameter("withdrawRateType");
        String withdrawRate = req.getParameter("withdrawRate");
        String reserveDays = req.getParameter("reserveDays");
        String reservedPhoneNo = req.getParameter("reservedPhoneNo");

        BankCardInfo bankCardInfo = new BankCardInfo();
        bankCardInfo.setBankName(bankName);
        bankCardInfo.setBankBranchName(bankBranchName);
        bankCardInfo.setAccountName(accountName);
        bankCardInfo.setBankCardNo(bankCardNo);
        bankCardInfo.setCityCode(cityCode);
        bankCardInfo.setProvinceCode(provinceCode);
        bankCardInfo.setAccountType(accountType);
        bankCardInfo.setLiquidationType(liquidationType);
        bankCardInfo.setWithdrawRateType(withdrawRateType);
        bankCardInfo.setWithdrawRate(withdrawRate);
        bankCardInfo.setReserveDays(reserveDays);
        builder.setBankCardInfo(bankCardInfo);

        //desireAuthInfo_开户意愿核实信息
        String legalPersonName = req.getParameter("legalPersonName");
        String legalPersonIdNo = req.getParameter("legalPersonIdNo");
        String legalPersonPhoneNo = req.getParameter("legalPersonPhoneNo");

        DesireAuthInfo desireAuthInfo = new DesireAuthInfo();
        desireAuthInfo.setLegalPersonName(legalPersonName);
        desireAuthInfo.setLegalPersonIdNo(legalPersonIdNo);
        desireAuthInfo.setLegalPersonPhoneNo(legalPersonPhoneNo);
        builder.setDesireAuthInfo(desireAuthInfo);

        //certificateInfo_商户资质信息
        String certificateInfoLegalPersonName = req.getParameter("certificateInfoLegalPersonName");
        String profession = req.getParameter("profession");
        String certificateInfoLegalPersonPhone = req.getParameter("certificateInfoLegalPersonPhone");
        String legalPersonIdType = req.getParameter("legalPersonIdType");
        String certificateInfoLegalPersonIdNo = req.getParameter("certificateInfoLegalPersonIdNo");
        String idEffectiveDateStart = req.getParameter("idEffectiveDateStart");
        String idEffectiveDateEnd = req.getParameter("idEffectiveDateEnd");
        String cerNoType = req.getParameter("cerNoType");
        String cerNo = req.getParameter("cerNo");
        String sellingArea = req.getParameter("sellingArea");
        String staffSize = req.getParameter("staffSize");
        String tradingScenarios = req.getParameter("tradingScenarios");
        String webSite = req.getParameter("webSite");
        String webSiteName = req.getParameter("webSiteName");
        String icp = req.getParameter("icp");
        String appName = req.getParameter("appName");
        String wechatAppletName = req.getParameter("wechatAppletName");
        String testAccountInfo = req.getParameter("testAccountInfo");
        String businessLicensePath = req.getParameter("businessLicensePath");
        String openAccountPath = req.getParameter("openAccountPath");
        String legalIdCardProsPath = req.getParameter("legalIdCardProsPath");
        String legalIdCardConsPath = req.getParameter("legalIdCardConsPath");
        String desireAuthPath = req.getParameter("desireAuthPath");
        String holdingIdCardPath = req.getParameter("holdingIdCardPath");
        String webSitePath = req.getParameter("webSitePath");
        String wechatAppletPath = req.getParameter("wechatAppletPath");
        String appPath = req.getParameter("appPath");
        String legalPersonBankCardPath = req.getParameter("legalPersonBankCardPath");
        String desireAuthOtherPath = req.getParameter("desireAuthOtherPath");
        String otherCerPath = req.getParameter("otherCerPath");
        String storeHeaderPath = req.getParameter("storeHeaderPath");
        String storeIndoorPath = req.getParameter("storeIndoorPath");

        CertificateInfo certificateInfo = new CertificateInfo();
        certificateInfo.setLegalPersonName(certificateInfoLegalPersonName);
        certificateInfo.setProfession(profession);
        certificateInfo.setLegalPersonPhone(certificateInfoLegalPersonPhone);
        certificateInfo.setLegalPersonIdType(legalPersonIdType);
        certificateInfo.setLegalPersonIdNo(certificateInfoLegalPersonIdNo);
        certificateInfo.setIdEffectiveDateStart(idEffectiveDateStart);
        certificateInfo.setIdEffectiveDateEnd(idEffectiveDateEnd);
        certificateInfo.setCerNoType(cerNoType);
        certificateInfo.setCerNo(cerNo);
        certificateInfo.setSellingArea(sellingArea);
        certificateInfo.setStaffSize(staffSize);
        certificateInfo.setTradingScenarios(tradingScenarios);
        certificateInfo.setWebSite(webSite);
        certificateInfo.setWebSiteName(webSiteName);
        certificateInfo.setIcp(icp);
        certificateInfo.setAppName(appName);
        certificateInfo.setWechatAppletName(wechatAppletName);
        certificateInfo.setTestAccountInfo(testAccountInfo);
        certificateInfo.setBusinessLicensePath(businessLicensePath);
        certificateInfo.setOpenAccountPath(openAccountPath);
        certificateInfo.setLegalIdCardProsPath(legalIdCardProsPath);
        certificateInfo.setLegalIdCardConsPath(legalIdCardConsPath);
        certificateInfo.setDesireAuthPath(desireAuthPath);
        certificateInfo.setDesireAuthOtherPath(desireAuthOtherPath);
        certificateInfo.setHoldingIdCardPath(holdingIdCardPath);
        certificateInfo.setWebSitePath(webSitePath);
        certificateInfo.setWechatAppletPath(wechatAppletPath);
        certificateInfo.setAppPath(appPath);
        certificateInfo.setOtherCerPath(otherCerPath);
        certificateInfo.setLegalPersonBankCardPath(legalPersonBankCardPath);
        certificateInfo.setStoreHeaderPath(storeHeaderPath);
        certificateInfo.setStoreIndoorPath(storeIndoorPath);
        builder.setCertificateInfo(certificateInfo);

        //certificateContacts_最终受益人信息
        CertificateContacts certificateContact = new CertificateContacts();
        List<CertificateContacts> Cc = new ArrayList<>();
        String jsonCCStr = req.getParameter("certificateContact").trim();
        System.out.println("最终受益人信息:" + jsonCCStr);
        JSONArray certificateContactArray = JSONObject.parseArray(jsonCCStr);
        for (Object o : certificateContactArray) {
            certificateContact = JSONObject.parseObject(o.toString(), CertificateContacts.class);
            Cc.add(certificateContact);
            builder.setCertificateContacts(Cc);

        }

        //contractInfo_商户合同信息
        String receiverName = req.getParameter("receiverName");
        String receiverPhone = req.getParameter("receiverPhone");
        String receiverAddress = req.getParameter("receiverAddress");
        String contractType = req.getParameter("contractType");

        ContractInfo contractInfo = new ContractInfo();
        contractInfo.setReceiverAddress(receiverAddress);
        contractInfo.setReceiverName(receiverName);
        contractInfo.setReceiverPhone(receiverPhone);
        contractInfo.setContractType(contractType);
        builder.setContractInfo(contractInfo);

        //paymentProfiles_产品支付方式
        PaymentProfile paymentProfile = new PaymentProfile();
        List<PaymentProfile> paymentProfiles = new ArrayList<>();
        String jsonPPStr = req.getParameter("paymentProfile").trim();
        System.out.println("产品支付方式:" + jsonPPStr);
        JSONArray paymentProfileArray = JSONObject.parseArray(jsonPPStr);
        for (Object o : paymentProfileArray) {
            paymentProfile = JSONObject.parseObject(o.toString(), PaymentProfile.class);
            paymentProfiles.add(paymentProfile);
            builder.setPaymentProfiles(paymentProfiles);

        }

        //settlementProfile_商户结算配置信息
        String settleMode = req.getParameter("settleMode");
        String minAmount = req.getParameter("minAmount");
        String cycleType = req.getParameter("cycleType");
        String cycleData = req.getParameter("cycleData");

        SettlementProfile settle = new SettlementProfile();
        settle.setReserveDays(reserveDays);
        settle.setMinAmount(minAmount);
        settle.setSettleMode(settleMode);
        settle.setCycleType(cycleType);
        settle.setCycleData(cycleData);
        builder.setSettlementProfile(settle);

        final PrintWriter out = resp.getWriter();
        DeclarationExecuter executer = new DeclarationExecuter();

        try {
            executer.declare(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("请求成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
