package com.upay.demo.declaration;

import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.SubMerchantBalanceBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.SubMerchantBalanceExecuter;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 子商户汇总余额查询接口
 */
public class SubMerchantBalanceQueryServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

        String merchantId = req.getParameter("merchantId");
        String requestId = req.getParameter("requestId");
        String currency = req.getParameter("currency");
        String accountType = req.getParameter("accountType");
        String dateTime = req.getParameter("dateTime");

        SubMerchantBalanceBuilder builder = new SubMerchantBalanceBuilder(merchantId);
        builder.setRequestId(requestId).setCurrency(currency)
                .setAccountType(accountType).setDateTime(dateTime);

        final PrintWriter out = resp.getWriter();

        SubMerchantBalanceExecuter executer = new SubMerchantBalanceExecuter();

        try {

            executer.query(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }
            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
